package cn.com.qmth.scancentral.controller.admin;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.AsyncTaskExecutor;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.concurrent.model.Semaphore;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.ReentrantException;

import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.entity.OmrGroupEntity;
import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.enums.Stage;
import cn.com.qmth.scancentral.service.OmrGroupService;
import cn.com.qmth.scancentral.task.thread.OmrGroupBuildThread;
import cn.com.qmth.scancentral.task.thread.OmrGroupDeleteThread;
import cn.com.qmth.scancentral.task.thread.OmrGroupResetThread;
import cn.com.qmth.scancentral.vo.OmrGroupUpdateVo;
import cn.com.qmth.scancentral.vo.OmrGroupVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.sf.json.JSONObject;

@RestController
@Api(tags = "识别对照任务组接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/check/omr/group")
@Aac(strict = false, auth = true)
public class OmrGroupController extends BaseController {

    @Autowired
    private OmrGroupService groupService;

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private AsyncTaskExecutor taskExecutor;

    @ApiOperation(value = "查询识别对照任务列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public List<OmrGroupVo> list(@RequestParam Long examId) {
        return groupService.listByExamId(examId);
    }

    @ApiOperation(value = "创建修改识别对照任务组")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public JSONObject save(@RequestBody OmrGroupUpdateVo omrGroupUpdateVo) {
        OmrGroupEntity group = groupService.save(omrGroupUpdateVo.getId(), omrGroupUpdateVo.getExamId(),
                omrGroupUpdateVo.getConditionList(), getAccessUser().getId());
        JSONObject result = new JSONObject();
        result.accumulate("id", group.getId());
        result.accumulate("updateTime", System.currentTimeMillis());
        return result;
    }

    @ApiOperation(value = "修改识别对照任务组阶段")
    @RequestMapping(value = "/toggle", method = RequestMethod.POST)
    public JSONObject toggle(@RequestParam Long id) {
        groupService.updateStage(id, getAccessUser().getId());
        JSONObject result = new JSONObject();
        result.accumulate("stage", Stage.SECOND);
        result.accumulate("updateTime", System.currentTimeMillis());
        return result;
    }

    @ApiOperation(value = "识别对照任务组生成任务")
    @RequestMapping(value = "/build", method = RequestMethod.POST)
    public JSONObject build(@RequestParam Long id) {
        OmrGroupEntity group = groupService.getById(id);
        if (group == null) {
            throw new ParameterException("分组不存在");
        }
        if (group.getFixed()) {
            throw new ParameterException("固定分组不支持手动创建任务");
        }
        Semaphore semaphore = concurrentService.getSemaphore(LockType.OMR_GROUP_BUILD + "-" + id);
        if (semaphore.tryAcquire()) {
            taskExecutor.submit(new OmrGroupBuildThread(id, groupService, concurrentService, semaphore));
        } else {
            throw new ReentrantException("该分组数据操作繁忙，请稍后重试");
        }
        JSONObject result = new JSONObject();
        result.accumulate("id", id);
        result.accumulate("building", !semaphore.isAvailable());
        result.accumulate("updateTime", System.currentTimeMillis());
        return result;
    }

    @ApiOperation(value = "重置识别对照任务组")
    @RequestMapping(value = "/reset", method = RequestMethod.POST)
    public JSONObject reset(@RequestParam Long id) {
        OmrGroupEntity group = groupService.getById(id);
        if (group == null) {
            throw new ParameterException("分组不存在");
        }
        Semaphore semaphore = concurrentService.getSemaphore(LockType.OMR_GROUP_RESET + "-" + id);
        if (semaphore.tryAcquire()) {
            taskExecutor.submit(new OmrGroupResetThread(id, groupService, concurrentService, semaphore));
        } else {
            throw new ReentrantException("该分组数据操作繁忙，请稍后重试");
        }
        JSONObject result = new JSONObject();
        result.accumulate("id", id);
        result.accumulate("reseting", !semaphore.isAvailable());
        result.accumulate("updateTime", System.currentTimeMillis());
        return result;
    }

    @ApiOperation(value = "删除识别对照任务组")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public JSONObject delete(@RequestParam Long id) {
        OmrGroupEntity group = groupService.getById(id);
        if (group == null) {
            throw new ParameterException("分组不存在");
        }
        if (group.getFixed()) {
            throw new ParameterException("固定分组不能删除");
        }
        Semaphore semaphore = concurrentService.getSemaphore(LockType.OMR_GROUP_DELETE + "-" + id);
        if (semaphore.tryAcquire()) {
            taskExecutor.submit(new OmrGroupDeleteThread(id, groupService, concurrentService, semaphore));
        } else {
            throw new ReentrantException("该分组数据操作繁忙，请稍后重试");
        }
        JSONObject result = new JSONObject();
        result.accumulate("id", id);
        result.accumulate("deleting", !semaphore.isAvailable());
        result.accumulate("updateTime", System.currentTimeMillis());
        return result;
    }
}
